@@ -46,25 +46,29 @@
 static const HsaApiTable* hsaApiTable;
 static const CoreApiTable* coreApiTable;
 static const AmdExtTable* amdExtTable;
+static const ToolsApiTable* toolsApiTable;
 
 void hsa_table_interface_init(const HsaApiTable* apiTable) {
     hsaApiTable = apiTable;
     coreApiTable = apiTable->core_;
     amdExtTable = apiTable->amd_ext_;
+    toolsApiTable = apiTable->tools_;
 }
 
 const HsaApiTable* hsa_table_interface_get_table() {
   return hsaApiTable;
 }
 
-class Init {
- public:
-  Init() { rocr::core::LoadInitialHsaApiTable(); }
-};
-static Init LinkAtLoadOrFirstTranslationUnitAccess;
-
 // Pass through stub functions
-hsa_status_t HSA_API hsa_init() { return coreApiTable->hsa_init_fn(); }
+hsa_status_t HSA_API hsa_init() {
+  // We initialize the api tables here once more since the code above is prone to a
+  // link-time ordering condition: This compilation unit here may get its global
+  // variables initialized earlier than the global objects in other compilation units.
+  // In particular Init::Init may get called earlier than that the underlying hsa_api_table_
+  // object in hsa_api_trace.cpp has been initialized.
+  rocr::core::LoadInitialHsaApiTable();
+  return coreApiTable->hsa_init_fn();
+}
 
 hsa_status_t HSA_API hsa_shut_down() { return coreApiTable->hsa_shut_down_fn(); }
 
@@ -975,6 +979,28 @@ hsa_status_t HSA_API
                                      num_dep_signals, dep_signals, completion_signal);
 }
 
+// Mirrors Amd Extension Apis
+hsa_status_t HSA_API
+    hsa_amd_memory_async_copy_on_engine(void* dst, hsa_agent_t dst_agent, const void* src,
+                              hsa_agent_t src_agent, size_t size,
+                              uint32_t num_dep_signals,
+                              const hsa_signal_t* dep_signals,
+                              hsa_signal_t completion_signal,
+                              hsa_amd_sdma_engine_id_t engine_id,
+                              bool force_copy_on_sdma) {
+  return amdExtTable->hsa_amd_memory_async_copy_on_engine_fn(
+                                     dst, dst_agent, src, src_agent, size,
+                                     num_dep_signals, dep_signals, completion_signal,
+                                     engine_id, force_copy_on_sdma);
+}
+
+// Mirrors Amd Extension Apis
+hsa_status_t HSA_API
+    hsa_amd_memory_copy_engine_status(hsa_agent_t dst_agent, hsa_agent_t src_agent,
+                                      uint32_t *engine_ids_mask) {
+  return amdExtTable->hsa_amd_memory_copy_engine_status_fn(dst_agent, src_agent, engine_ids_mask);
+}
+
 // Mirrors Amd Extension Apis
 hsa_status_t HSA_API hsa_amd_memory_async_copy_rect(
     const hsa_pitched_ptr_t* dst, const hsa_dim3_t* dst_offset, const hsa_pitched_ptr_t* src,
@@ -1178,6 +1204,100 @@ hsa_status_t HSA_API hsa_amd_svm_prefetch_async(void* ptr, size_t size, hsa_agen
   return amdExtTable->hsa_amd_svm_prefetch_async_fn(ptr, size, agent, num_dep_signals, dep_signals, completion_signal);
 }
 
+// Mirrors Amd Extension Apis
+hsa_status_t HSA_API hsa_amd_spm_acquire(hsa_agent_t agent) {
+  return amdExtTable->hsa_amd_spm_acquire_fn(agent);
+}
+
+// Mirrors Amd Extension Apis
+hsa_status_t HSA_API hsa_amd_spm_release(hsa_agent_t agent) {
+  return amdExtTable->hsa_amd_spm_release_fn(agent);
+}
+
+// Mirrors Amd Extension Apis
+hsa_status_t HSA_API hsa_amd_spm_set_dest_buffer(hsa_agent_t agent, size_t size, uint32_t* timeout,
+                                                 uint32_t* size_copied, void* dest,
+                                                 bool* is_data_loss) {
+  return amdExtTable->hsa_amd_spm_set_dest_buffer_fn(agent, size, timeout, size_copied, dest,
+                                                     is_data_loss);
+}
+
+// Mirrors Amd Extension Apis
+hsa_status_t HSA_API hsa_amd_portable_export_dmabuf(const void* ptr, size_t size, int* dmabuf,
+                                                    uint64_t* offset) {
+  return amdExtTable->hsa_amd_portable_export_dmabuf_fn(ptr, size, dmabuf, offset);
+}
+
+// Mirrors Amd Extension Apis
+hsa_status_t HSA_API hsa_amd_portable_close_dmabuf(int dmabuf) {
+  return amdExtTable->hsa_amd_portable_close_dmabuf_fn(dmabuf);
+}
+
+hsa_status_t HSA_API hsa_amd_vmem_address_reserve(void** ptr, size_t size, uint64_t address,
+                                                  uint64_t flags) {
+  return amdExtTable->hsa_amd_vmem_address_reserve_fn(ptr, size, address, flags);
+}
+
+hsa_status_t HSA_API hsa_amd_vmem_address_free(void* ptr, size_t size) {
+  return amdExtTable->hsa_amd_vmem_address_free_fn(ptr, size);
+}
+
+hsa_status_t HSA_API hsa_amd_vmem_handle_create(hsa_amd_memory_pool_t pool, size_t size,
+                                                hsa_amd_memory_type_t type, uint64_t flags,
+                                                hsa_amd_vmem_alloc_handle_t* memory_handle) {
+  return amdExtTable->hsa_amd_vmem_handle_create_fn(pool, size, type, flags, memory_handle);
+}
+
+hsa_status_t HSA_API hsa_amd_vmem_handle_release(hsa_amd_vmem_alloc_handle_t memory_handle) {
+  return amdExtTable->hsa_amd_vmem_handle_release_fn(memory_handle);
+}
+
+hsa_status_t HSA_API hsa_amd_vmem_map(void* va, size_t size, size_t in_offset,
+                                      hsa_amd_vmem_alloc_handle_t memory_handle, uint64_t flags) {
+  return amdExtTable->hsa_amd_vmem_map_fn(va, size, in_offset, memory_handle, flags);
+}
+
+hsa_status_t HSA_API hsa_amd_vmem_unmap(void* va, size_t size) {
+  return amdExtTable->hsa_amd_vmem_unmap_fn(va, size);
+}
+
+hsa_status_t HSA_API hsa_amd_vmem_set_access(void* va, size_t size,
+                                             const hsa_amd_memory_access_desc_t* desc,
+                                             const size_t desc_cnt) {
+  return amdExtTable->hsa_amd_vmem_set_access_fn(va, size, desc, desc_cnt);
+}
+
+hsa_status_t HSA_API hsa_amd_vmem_get_access(void* va, hsa_access_permission_t* perms,
+                                             const hsa_agent_t agent_handle) {
+  return amdExtTable->hsa_amd_vmem_get_access_fn(va, perms, agent_handle);
+}
+
+hsa_status_t HSA_API hsa_amd_vmem_export_shareable_handle(int* dmabuf_fd,
+                                                          hsa_amd_vmem_alloc_handle_t handle,
+                                                          uint64_t flags) {
+  return amdExtTable->hsa_amd_vmem_export_shareable_handle_fn(dmabuf_fd, handle, flags);
+}
+
+hsa_status_t HSA_API hsa_amd_vmem_import_shareable_handle(int dmabuf_fd,
+                                                          hsa_amd_vmem_alloc_handle_t* handle) {
+  return amdExtTable->hsa_amd_vmem_import_shareable_handle_fn(dmabuf_fd, handle);
+}
+
+hsa_status_t HSA_API hsa_amd_vmem_retain_alloc_handle(hsa_amd_vmem_alloc_handle_t* handle,
+                                                      void* addr) {
+  return amdExtTable->hsa_amd_vmem_retain_alloc_handle_fn(handle, addr);
+}
+
+hsa_status_t HSA_API hsa_amd_vmem_get_alloc_properties_from_handle(
+    hsa_amd_vmem_alloc_handle_t alloc_handle, hsa_amd_memory_pool_t* pool,
+    hsa_amd_memory_type_t* type) {
+  return amdExtTable->hsa_amd_vmem_get_alloc_properties_from_handle_fn(alloc_handle, pool, type);
+}
+
+hsa_status_t HSA_API hsa_amd_agent_set_async_scratch_limit(hsa_agent_t agent, size_t threshold) {
+  return amdExtTable->hsa_amd_agent_set_async_scratch_limit_fn(agent, threshold);
+}
+
 // Tools only table interfaces.
 namespace rocr {
 
