@@ -100,7 +100,7 @@ class MemoryRegion : public core::MemoryRegion {
 
   ~MemoryRegion();
 
-  hsa_status_t Allocate(size_t& size, AllocateFlags alloc_flags, void** address) const;
+  hsa_status_t Allocate(size_t& size, AllocateFlags alloc_flags, void** address, int agent_node_id = 0) const;
 
   hsa_status_t Free(void* address, size_t size) const;
 
@@ -173,7 +173,7 @@ class MemoryRegion : public core::MemoryRegion {
     return static_cast<uint32_t>(mem_props_.MemoryClockMax);
   }
 
-  __forceinline bool extended_scope_fine_grain() const { return extended_scope_fine_grain_; }
+  __forceinline size_t GetPageSize() const { return kPageSize_; }
 
  private:
   const HsaMemoryProperties mem_props_;
@@ -184,9 +184,6 @@ class MemoryRegion : public core::MemoryRegion {
 
   size_t max_single_alloc_size_;
 
-  // Enables creating an extended scope fine grained memory pool region
-  const bool extended_scope_fine_grain_;
-
   // Used to collect total system memory
   static size_t max_sysmem_alloc_size_;
 
@@ -203,7 +200,7 @@ class MemoryRegion : public core::MemoryRegion {
                                              const core::Runtime::LinkInfo& link_info) const;
 
   // Operational body for Allocate.  Recursive.
-  hsa_status_t AllocateImpl(size_t& size, AllocateFlags alloc_flags, void** address) const;
+  hsa_status_t AllocateImpl(size_t& size, AllocateFlags alloc_flags, void** address, int agent_node_id) const;
 
   // Operational body for Free.  Recursive.
   hsa_status_t FreeImpl(void* address, size_t size) const;
