@@ -51,6 +51,12 @@
 #include <tuple>
 #include <utility>
 #include <thread>
+#include <sys/un.h>
+
+#if defined(__linux__)
+#include <xf86drm.h>
+#include <amdgpu.h>
+#endif
 
 #include "core/inc/hsa_ext_interface.h"
 #include "core/inc/hsa_internal.h"
@@ -119,6 +125,7 @@ class Runtime {
     HsaVersionInfo version;
     bool supports_exception_debugging;
     bool supports_event_age;
+    bool supports_core_dump;
   };
 
   /// @brief Open connection to kernel driver and increment reference count.
@@ -131,6 +138,9 @@ class Runtime {
   /// @retval True if the connection to kernel driver is opened.
   static bool IsOpen();
 
+  // @brief Callback handler for HW Exceptions.
+  static bool HwExceptionHandler(hsa_signal_value_t val, void* arg);
+
   // @brief Callback handler for VM fault access.
   static bool VMFaultHandler(hsa_signal_value_t val, void* arg);
 
@@ -188,7 +198,7 @@ class Runtime {
   /// @retval ::HSA_STATUS_SUCCESS If allocation is successful.
   hsa_status_t AllocateMemory(const MemoryRegion* region, size_t size,
                               MemoryRegion::AllocateFlags alloc_flags,
-                              void** address);
+                              void** address, int agent_node_id = 0);
 
   /// @brief Free memory previously allocated with AllocateMemory.
   ///
@@ -349,6 +359,40 @@ class Runtime {
 
   hsa_status_t DmaBufClose(int dmabuf);
 
+  hsa_status_t VMemoryAddressReserve(void** ptr, size_t size, uint64_t address, uint64_t flags);
+
+  hsa_status_t VMemoryAddressFree(void* ptr, size_t size);
+
+  hsa_status_t VMemoryHandleCreate(const MemoryRegion* region, size_t size,
+                                   MemoryRegion::AllocateFlags alloc_flags,
+                                   uint64_t flags, hsa_amd_vmem_alloc_handle_t* memoryHandle);
+
+  hsa_status_t VMemoryHandleRelease(hsa_amd_vmem_alloc_handle_t memoryHandle);
+
+  hsa_status_t VMemoryHandleMap(void* va, size_t size, size_t in_offset,
+                                hsa_amd_vmem_alloc_handle_t memoryHandle, uint64_t flags);
+
+  hsa_status_t VMemoryHandleUnmap(void* va, size_t size);
+
+  hsa_status_t VMemorySetAccess(void* va, size_t size, const hsa_amd_memory_access_desc_t* desc,
+                                size_t desc_cnt);
+
+  hsa_status_t VMemoryGetAccess(const void* va, hsa_access_permission_t* perms,
+                                hsa_agent_t agent_handle);
+
+  hsa_status_t VMemoryExportShareableHandle(int* dmabuf_fd,
+                                            const hsa_amd_vmem_alloc_handle_t handle,
+                                            const uint64_t flags);
+
+  hsa_status_t VMemoryImportShareableHandle(const int dmabuf_fd,
+                                            hsa_amd_vmem_alloc_handle_t* handle);
+
+  hsa_status_t VMemoryRetainAllocHandle(hsa_amd_vmem_alloc_handle_t* memoryHandle, void* addr);
+
+  hsa_status_t VMemoryGetAllocPropertiesFromHandle(const hsa_amd_vmem_alloc_handle_t memoryHandle,
+                                                   const core::MemoryRegion** mem_region,
+                                                   hsa_amd_memory_type_t* type);
+
   const std::vector<Agent*>& cpu_agents() { return cpu_agents_; }
 
   const std::vector<Agent*>& gpu_agents() { return gpu_agents_; }
@@ -375,7 +419,7 @@ class Runtime {
 
   amd::hsa::code::AmdHsaCodeManager* code_manager() { return &code_manager_; }
 
-  std::function<void*(size_t size, size_t align, MemoryRegion::AllocateFlags flags)>&
+  std::function<void*(size_t size, size_t align, MemoryRegion::AllocateFlags flags, int agent_node_id)>&
   system_allocator() {
     return system_allocator_;
   }
@@ -409,14 +453,20 @@ class Runtime {
       kfd_version.supports_event_age = true;
   }
 
-  void KfdVersion(bool exception_debugging) {
+  void KfdVersion(bool exception_debugging, bool core_dump) {
     kfd_version.supports_exception_debugging = exception_debugging;
+    kfd_version.supports_core_dump = core_dump;
   }
 
   KfdVersion_t KfdVersion() const { return kfd_version; }
 
+  bool VirtualMemApiSupported() const { return virtual_mem_api_supported_; }
+  bool XnackEnabled() const { return xnack_enabled_; }
+  void XnackEnabled(bool enable) { xnack_enabled_ = enable; }
+
  protected:
   static void AsyncEventsLoop(void*);
+  static void AsyncIPCSockServerConnLoop(void*);
 
   struct AllocationRegion {
     AllocationRegion()
@@ -424,7 +474,8 @@ class Runtime {
           size(0),
           size_requested(0),
           alloc_flags(core::MemoryRegion::AllocateNoFlags),
-          user_ptr(nullptr) {}
+          user_ptr(nullptr),
+          ldrm_bo(NULL) {}
     AllocationRegion(const MemoryRegion* region_arg, size_t size_arg, size_t size_requested,
                      MemoryRegion::AllocateFlags alloc_flags)
         : region(region_arg),
@@ -445,6 +496,7 @@ class Runtime {
     MemoryRegion::AllocateFlags alloc_flags;
     void* user_ptr;
     std::unique_ptr<std::vector<notifier_t>> notifiers;
+    amdgpu_bo_handle ldrm_bo;
   };
 
   struct AsyncEventsControl {
@@ -453,7 +505,7 @@ class Runtime {
 
     hsa_signal_t wake;
     os::Thread async_events_thread_;
-    KernelMutex lock;
+    HybridMutex lock;
     bool exit;
   };
 
@@ -534,8 +586,8 @@ class Runtime {
   /// @brief Close tool libraries.
   void CloseTools();
 
-  // @brief Binds virtual memory access fault handler to this node.
-  void BindVmFaultHandler();
+  // @brief Binds Error handlers to this node.
+  void BindErrorHandlers();
 
   // @brief Acquire snapshot of system event handlers.
   // Returns a copy to avoid holding a lock during callbacks.
@@ -607,7 +659,7 @@ class Runtime {
   prefetch_map_t prefetch_map_;
 
   // Allocator using ::system_region_
-  std::function<void*(size_t size, size_t align, MemoryRegion::AllocateFlags flags)> system_allocator_;
+  std::function<void*(size_t size, size_t align, MemoryRegion::AllocateFlags flags, int agent_node_id)> system_allocator_;
 
   // Deallocator using ::system_region_
   std::function<void(void*)> system_deallocator_;
@@ -633,6 +685,12 @@ class Runtime {
   // @brief HSA signal to contain the VM fault event.
   Signal* vm_fault_signal_;
 
+  // @brief AMD HSA event to monitor for HW exceptions.
+  HsaEvent* hw_exception_event_;
+
+  // @brief HSA signal to contain the HW exceptionevent.
+  Signal* hw_exception_signal_;
+
   // Custom system event handlers.
   std::vector<std::pair<AMD::callback_t<hsa_amd_system_event_callback_t>, void*>>
       system_event_handlers_;
@@ -662,10 +720,120 @@ class Runtime {
 
   std::unique_ptr<AMD::SvmProfileControl> svm_profile_;
 
+  // IPC DMA buf unix domain socket server dmabuf FD passing
+  int ipc_sock_server_fd_;
+  std::map<uint64_t, std::pair<void*, size_t>> ipc_sock_server_conns_;
+  KernelMutex ipc_sock_server_lock_;
+
+ private:
+  void CheckVirtualMemApiSupport();
+  int GetAmdgpuDeviceArgs(Agent* agent, amdgpu_bo_handle bo, int* drm_fd, uint64_t* cpu_addr);
+
+  bool virtual_mem_api_supported_;
+  bool xnack_enabled_;
+
+  typedef void* ThunkHandle;
+
+  struct AddressHandle {
+    AddressHandle() : size(0), use_count(0) {}
+    AddressHandle(size_t size) : size(size), use_count(0) {}
+
+    size_t size;
+    int use_count;
+  };
+  std::map<const void*, AddressHandle> reserved_address_map_;  // Indexed by VA
+
+  struct MemoryHandle {
+    MemoryHandle() : region(NULL), size(0), ref_count(0), thunk_handle(NULL), alloc_flag(0) {}
+    MemoryHandle(const MemoryRegion* region, size_t size, uint64_t flags_unused,
+                 ThunkHandle thunk_handle, MemoryRegion::AllocateFlags alloc_flag)
+        : region(region),
+          size(size),
+          ref_count(1),
+          use_count(0),
+          thunk_handle(thunk_handle),
+          alloc_flag(alloc_flag) {}
+
+    static __forceinline hsa_amd_vmem_alloc_handle_t Convert(void* handle) {
+      hsa_amd_vmem_alloc_handle_t ret_handle = {
+          static_cast<uint64_t>(reinterpret_cast<uintptr_t>(handle))};
+      return ret_handle;
+    }
+
+    __forceinline core::Agent* agentOwner() const { return region->owner(); }
+
+    const MemoryRegion* region;
+    size_t size;
+    int ref_count;
+    int use_count;
+    ThunkHandle thunk_handle;  // handle returned by hsaKmtAllocMemory(NoAddress = 1)
+    MemoryRegion::AllocateFlags alloc_flag;
+  };
+  std::map<ThunkHandle, MemoryHandle> memory_handle_map_;
+
+  struct MappedHandle;
+  struct MappedHandleAllowedAgent {
+    MappedHandleAllowedAgent()
+        : va(NULL), permissions(HSA_ACCESS_PERMISSION_NONE), mappedHandle(NULL), ldrm_bo(0) {}
+    MappedHandleAllowedAgent(MappedHandle* _mappedHandle, Agent* targetAgent, void* va, size_t size,
+                             hsa_access_permission_t perms);
+    ~MappedHandleAllowedAgent();
+
+    hsa_status_t RemoveAccess();
+    hsa_status_t EnableAccess(hsa_access_permission_t perms);
+
+    void* va;
+    size_t size;
+    Agent* targetAgent;
+    hsa_access_permission_t permissions;
+    MappedHandle* mappedHandle;
+    amdgpu_bo_handle ldrm_bo;
+  };
+
+  struct MappedHandle {
+    MappedHandle()
+        : mem_handle(NULL),
+          address_handle(NULL),
+          offset(0),
+          mmap_offset(0),
+          size(0),
+          drm_fd(-1),
+          drm_cpu_addr(NULL),
+          ldrm_bo(0) {}
+
+    MappedHandle(MemoryHandle* mem_handle, AddressHandle* address_handle, uint64_t offset,
+                 size_t size, int drm_fd, void* drm_cpu_addr, hsa_access_permission_t perm,
+                 amdgpu_bo_handle bo)
+        : mem_handle(mem_handle),
+          address_handle(address_handle),
+          offset(offset),
+          mmap_offset(0),
+          size(size),
+          drm_fd(drm_fd),
+          drm_cpu_addr(drm_cpu_addr),
+          ldrm_bo(bo) {}
+
+    __forceinline core::Agent* agentOwner() const { return mem_handle->region->owner(); }
+
+    MemoryHandle* mem_handle;
+    AddressHandle* address_handle;
+    uint64_t offset;
+    uint64_t mmap_offset;
+    size_t size;
+    int drm_fd;
+    void* drm_cpu_addr;  // CPU Buffer address
+    amdgpu_bo_handle ldrm_bo;
+    std::map<Agent*, MappedHandleAllowedAgent> allowed_agents;
+  };
+  std::map<const void*, MappedHandle> mapped_handle_map_;  // Indexed by VA
+
   // Frees runtime memory when the runtime library is unloaded if safe to do so.
   // Failure to release the runtime indicates an incorrect application but is
   // common (example: calls library routines at process exit).
   friend class RuntimeCleanup;
+
+  void InitIPCDmaBufSupport();
+  bool ipc_dmabuf_supported_;
 };
 
 }  // namespace core
