@@ -157,10 +157,13 @@ static_assert((offsetof(SharedSignal, sdma_end_ts) % 32) == 0,
 static_assert(sizeof(SharedSignal) == 128,
               "Bad SharedSignal size.");
 
+
+#define SIGNAL_PREALLOC_BLOCKS 512 //16K Signals
+
 /// @brief Pool class for SharedSignal suitable for use with Shared.
 class SharedSignalPool_t : private BaseShared {
  public:
-  SharedSignalPool_t() : block_size_(minblock_) {}
+  SharedSignalPool_t() : block_size_(SIGNAL_PREALLOC_BLOCKS * minblock_) {}
   ~SharedSignalPool_t() { clear(); }
 
   SharedSignal* alloc();
@@ -169,7 +172,7 @@ class SharedSignalPool_t : private BaseShared {
 
  private:
   static const size_t minblock_ = 4096 / sizeof(SharedSignal);
-  KernelMutex lock_;
+  HybridMutex lock_;
   std::vector<SharedSignal*> free_list_;
   std::vector<std::pair<void*, size_t>> block_list_;
   size_t block_size_;
@@ -404,7 +407,7 @@ class Signal {
   core::Agent* async_copy_agent_;
 
  private:
-  static KernelMutex ipcLock_;
+  static HybridMutex ipcLock_;
   static std::map<decltype(hsa_signal_t::handle), Signal*> ipcMap_;
 
   static Signal* lookupIpc(hsa_signal_t signal);
