@@ -491,6 +491,388 @@ static std::string kBlitKernelSource(R"(
   L_FILL_PHASE_2_DONE:
     s_endpgm
   end
+
+shader CopyAligned_940
+  type(CS)
+  user_sgpr_count(2)
+  sgpr_count(32)
+  vgpr_count(8 + (kCopyAlignedUnroll * kCopyAlignedVecWidth))
+
+  // Retrieve kernel arguments.
+  s_load_dwordx4          s[4:7], s[0:1], s_load_dword_offset(0x0)
+  s_load_dwordx4          s[8:11], s[0:1], s_load_dword_offset(0x10)
+  s_load_dwordx4          s[12:15], s[0:1], s_load_dword_offset(0x20)
+  s_load_dwordx4          s[16:19], s[0:1], s_load_dword_offset(0x30)
+  s_load_dwordx4          s[20:23], s[0:1], s_load_dword_offset(0x40)
+  s_load_dword            s24, s[0:1], s_load_dword_offset(0x50)
+  s_waitcnt               lgkmcnt(0)
+
+  // Compute workitem id.
+  s_lshl_b32              s2, s2, 0x6
+  v_add_co_u32            v0, vcc, s2, v0
+
+  // =====================================================
+  // Phase 1: Byte copy up to 0x100 destination alignment.
+  // =====================================================
+
+  // Compute phase source address.
+  v_mov_b32               v3, s5
+  v_add_co_u32            v2, vcc, v0, s4
+  v_addc_co_u32           v3, vcc, v3, 0x0, vcc
+
+  // Compute phase destination address.
+  v_mov_b32               v5, s7
+  v_add_co_u32            v4, vcc, v0, s6
+  v_addc_co_u32           v5, vcc, v5, 0x0, vcc
+
+L_COPY_ALIGNED_PHASE_1_LOOP:
+  // Mask off lanes (or branch out) after phase end.
+  v_cmp_lt_u64            vcc, v[2:3], s[8:9]
+  s_cbranch_vccz          L_COPY_ALIGNED_PHASE_1_DONE
+  s_and_b64               exec, exec, vcc
+
+  // Load from/advance the source address.
+  flat_load_ubyte         v1, v[2:3] sc0:1 sc1:1
+  s_waitcnt               vmcnt(0)
+  v_add_co_u32            v2, vcc, v2, s24
+  v_addc_co_u32           v3, vcc, v3, 0x0, vcc
+
+  // Write to/advance the destination address.
+  flat_store_byte         v[4:5], v1 sc0:1 sc1:1
+  v_add_co_u32            v4, vcc, v4, s24
+  v_addc_co_u32           v5, vcc, v5, 0x0, vcc
+
+  // Repeat until branched out.
+  s_branch                L_COPY_ALIGNED_PHASE_1_LOOP
+
+L_COPY_ALIGNED_PHASE_1_DONE:
+  // Restore EXEC mask for all lanes.
+  s_mov_b64               exec, 0xFFFFFFFFFFFFFFFF
+
+  // ========================================================
+  // Phase 2: Unrolled dword[x4] copy up to last whole block.
+  // ========================================================
+
+  // Compute unrolled dword[x4] stride across all threads.
+  if kCopyAlignedVecWidth == 4
+    s_lshl_b32            s25, s24, 0x4
+  else
+    s_lshl_b32            s25, s24, 0x2
+  end
+
+  // Compute phase source address.
+  if kCopyAlignedVecWidth == 4
+    v_lshlrev_b32         v1, 0x4, v0
+  else
+    v_lshlrev_b32         v1, 0x2, v0
+  end
+
+  v_mov_b32               v3, s9
+  v_add_co_u32            v2, vcc, v1, s8
+  v_addc_co_u32           v3, vcc, v3, 0x0, vcc
+
+  // Compute phase destination address.
+  v_mov_b32               v5, s11
+  v_add_co_u32            v4, vcc, v1, s10
+  v_addc_co_u32           v5, vcc, v5, 0x0, vcc
+
+L_COPY_ALIGNED_PHASE_2_LOOP:
+  // Branch out after phase end.
+  v_cmp_lt_u64            vcc, v[2:3], s[12:13]
+  s_cbranch_vccz          L_COPY_ALIGNED_PHASE_2_DONE
+
+  // Load from/advance the source address.
+  for var i = 0; i < kCopyAlignedUnroll; i ++
+    if kCopyAlignedVecWidth == 4
+      flat_load_dwordx4   v[8 + (i * 4)], v[2:3] sc0:1 sc1:1
+    else
+      flat_load_dword     v[8 + i], v[2:3] sc0:1 sc1:1
+    end
+
+    v_add_co_u32             v2, vcc, v2, s25
+    v_addc_co_u32         v3, vcc, v3, 0x0, vcc
+  end
+
+  // Write to/advance the destination address.
+  s_waitcnt               vmcnt(0)
+
+  for var i = 0; i < kCopyAlignedUnroll; i ++
+    if kCopyAlignedVecWidth == 4
+      flat_store_dwordx4  v[4:5], v[8 + (i * 4)] sc0:1 sc1:1
+    else
+      flat_store_dword    v[4:5], v[8 + i] sc0:1 sc1:1
+    end
+
+    v_add_co_u32             v4, vcc, v4, s25
+    v_addc_co_u32         v5, vcc, v5, 0x0, vcc
+  end
+
+  // Repeat until branched out.
+  s_branch                L_COPY_ALIGNED_PHASE_2_LOOP
+
+L_COPY_ALIGNED_PHASE_2_DONE:
+
+  // ===========================================
+  // Phase 3: Dword copy up to last whole dword.
+  // ===========================================
+
+  // Compute dword stride across all threads.
+  s_lshl_b32              s25, s24, 0x2
+
+  // Compute phase source address.
+  v_lshlrev_b32           v1, 0x2, v0
+  v_mov_b32               v3, s13
+  v_add_co_u32            v2, vcc, v1, s12
+  v_addc_co_u32           v3, vcc, v3, 0x0, vcc
+
+  // Compute phase destination address.
+  v_mov_b32               v5, s15
+  v_add_co_u32            v4, vcc, v1, s14
+  v_addc_co_u32           v5, vcc, v5, 0x0, vcc
+
+L_COPY_ALIGNED_PHASE_3_LOOP:
+  // Mask off lanes (or branch out) after phase end.
+  v_cmp_lt_u64            vcc, v[2:3], s[16:17]
+  s_cbranch_vccz          L_COPY_ALIGNED_PHASE_3_DONE
+  s_and_b64               exec, exec, vcc
+
+  // Load from/advance the source address.
+  flat_load_dword         v1, v[2:3] sc0:1 sc1:1
+  v_add_co_u32            v2, vcc, v2, s25
+  v_addc_co_u32           v3, vcc, v3, 0x0, vcc
+  s_waitcnt               vmcnt(0)
+
+  // Write to/advance the destination address.
+  flat_store_dword        v[4:5], v1 sc0:1 sc1:1
+  v_add_co_u32            v4, vcc, v4, s25
+  v_addc_co_u32           v5, vcc, v5, 0x0, vcc
+
+  // Repeat until branched out.
+  s_branch                L_COPY_ALIGNED_PHASE_3_LOOP
+
+L_COPY_ALIGNED_PHASE_3_DONE:
+  // Restore EXEC mask for all lanes.
+  s_mov_b64               exec, 0xFFFFFFFFFFFFFFFF
+
+  // =============================
+  // Phase 4: Byte copy up to end.
+  // =============================
+
+  // Compute phase source address.
+  v_mov_b32               v3, s17
+  v_add_co_u32            v2, vcc, v0, s16
+  v_addc_co_u32           v3, vcc, v3, 0x0, vcc
+
+  // Compute phase destination address.
+  v_mov_b32               v5, s19
+  v_add_co_u32            v4, vcc, v0, s18
+  v_addc_co_u32           v5, vcc, v5, 0x0, vcc
+
+  // Mask off lanes (or branch out) after phase end.
+  v_cmp_lt_u64            vcc, v[2:3], s[20:21]
+  s_cbranch_vccz          L_COPY_ALIGNED_PHASE_4_DONE
+  s_and_b64               exec, exec, vcc
+
+  // Load from the source address.
+  flat_load_ubyte         v1, v[2:3] sc0:1 sc1:1
+  s_waitcnt               vmcnt(0)
+
+  // Write to the destination address.
+  flat_store_byte         v[4:5], v1 sc0:1 sc1:1
+
+L_COPY_ALIGNED_PHASE_4_DONE:
+  s_endpgm
+end
+
+shader CopyMisaligned_940
+  type(CS)
+  user_sgpr_count(2)
+  sgpr_count(23)
+  vgpr_count(6 + kCopyMisalignedUnroll)
+
+  // Retrieve kernel arguments.
+  s_load_dwordx4          s[4:7], s[0:1], s_load_dword_offset(0x0)
+  s_load_dwordx4          s[8:11], s[0:1], s_load_dword_offset(0x10)
+  s_load_dwordx4          s[12:15], s[0:1], s_load_dword_offset(0x20)
+  s_load_dword            s16, s[0:1], s_load_dword_offset(0x30)
+  s_waitcnt               lgkmcnt(0)
+
+  // Compute workitem id.
+  s_lshl_b32              s2, s2, 0x6
+  v_add_co_u32            v0, vcc, s2, v0
+
+  // ===================================================
+  // Phase 1: Unrolled byte copy up to last whole block.
+  // ===================================================
+
+  // Compute phase source address.
+  v_mov_b32               v3, s5
+  v_add_co_u32            v2, vcc, v0, s4
+  v_addc_co_u32           v3, vcc, v3, 0x0, vcc
+
+  // Compute phase destination address.
+  v_mov_b32               v5, s7
+  v_add_co_u32            v4, vcc, v0, s6
+  v_addc_co_u32           v5, vcc, v5, 0x0, vcc
+
+L_COPY_MISALIGNED_PHASE_1_LOOP:
+  // Branch out after phase end.
+  v_cmp_lt_u64            vcc, v[2:3], s[8:9]
+  s_cbranch_vccz          L_COPY_MISALIGNED_PHASE_1_DONE
+
+  // Load from/advance the source address.
+  for var i = 0; i < kCopyMisalignedUnroll; i ++
+    flat_load_ubyte       v[6 + i], v[2:3] sc0:1 sc1:1
+    v_add_co_u32          v2, vcc, v2, s16
+    v_addc_co_u32         v3, vcc, v3, 0x0, vcc
+  end
+
+  // Write to/advance the destination address.
+  s_waitcnt               vmcnt(0)
+
+  for var i = 0; i < kCopyMisalignedUnroll; i ++
+    flat_store_byte       v[4:5], v[6 + i] sc0:1 sc1:1
+    v_add_co_u32          v4, vcc, v4, s16
+    v_addc_co_u32         v5, vcc, v5, 0x0, vcc
+  end
+
+  // Repeat until branched out.
+  s_branch                L_COPY_MISALIGNED_PHASE_1_LOOP
+
+L_COPY_MISALIGNED_PHASE_1_DONE:
+
+  // =============================
+  // Phase 2: Byte copy up to end.
+  // =============================
+
+  // Compute phase source address.
+  v_mov_b32               v3, s9
+  v_add_co_u32            v2, vcc, v0, s8
+  v_addc_co_u32           v3, vcc, v3, 0x0, vcc
+
+  // Compute phase destination address.
+  v_mov_b32               v5, s11
+  v_add_co_u32            v4, vcc, v0, s10
+  v_addc_co_u32           v5, vcc, v5, 0x0, vcc
+
+L_COPY_MISALIGNED_PHASE_2_LOOP:
+  // Mask off lanes (or branch out) after phase end.
+  v_cmp_lt_u64            vcc, v[2:3], s[12:13]
+  s_cbranch_vccz          L_COPY_MISALIGNED_PHASE_2_DONE
+  s_and_b64               exec, exec, vcc
+
+  // Load from/advance the source address.
+  flat_load_ubyte         v1, v[2:3] sc0:1 sc1:1
+  v_add_co_u32            v2, vcc, v2, s16
+  v_addc_co_u32           v3, vcc, v3, 0x0, vcc
+  s_waitcnt               vmcnt(0)
+
+  // Write to/advance the destination address.
+  flat_store_byte         v[4:5], v1 sc0:1 sc1:1
+  v_add_co_u32            v4, vcc, v4, s16
+  v_addc_co_u32           v5, vcc, v5, 0x0, vcc
+
+  // Repeat until branched out.
+  s_branch                L_COPY_MISALIGNED_PHASE_2_LOOP
+
+L_COPY_MISALIGNED_PHASE_2_DONE:
+  s_endpgm
+end
+
+shader Fill_940
+  type(CS)
+  user_sgpr_count(2)
+  sgpr_count(19)
+  vgpr_count(8)
+
+  // Retrieve kernel arguments.
+  s_load_dwordx4          s[4:7], s[0:1], s_load_dword_offset(0x0)
+  s_load_dwordx4          s[8:11], s[0:1], s_load_dword_offset(0x10)
+  s_waitcnt               lgkmcnt(0)
+
+  // Compute workitem id.
+  s_lshl_b32              s2, s2, 0x6
+  v_add_co_u32            v0, vcc, s2, v0
+
+  // Copy fill pattern into VGPRs.
+  for var i = 0; i < kFillVecWidth; i ++
+    v_mov_b32           v[4 + i], s10
+  end
+
+  // ========================================================
+  // Phase 1: Unrolled dword[x4] fill up to last whole block.
+  // ========================================================
+
+  // Compute unrolled dword[x4] stride across all threads.
+  if kFillVecWidth == 4
+    s_lshl_b32            s12, s11, 0x4
+  else
+    s_lshl_b32            s12, s11, 0x2
+  end
+
+  // Compute phase destination address.
+  if kFillVecWidth == 4
+    v_lshlrev_b32         v1, 0x4, v0
+  else
+    v_lshlrev_b32         v1, 0x2, v0
+  end
+
+  v_mov_b32               v3, s5
+  v_add_co_u32            v2, vcc, v1, s4
+  v_addc_co_u32           v3, vcc, v3, 0x0, vcc
+
+L_FILL_PHASE_1_LOOP:
+  // Branch out after phase end.
+  v_cmp_lt_u64            vcc, v[2:3], s[6:7]
+  s_cbranch_vccz          L_FILL_PHASE_1_DONE
+
+  // Write to/advance the destination address.
+  for var i = 0; i < kFillUnroll; i ++
+    if kFillVecWidth == 4
+      flat_store_dwordx4  v[2:3], v[4:7] sc0:1 sc1:1
+    else
+      flat_store_dword    v[2:3], v4 sc0:1 sc1:1
+    end
+
+    v_add_co_u32          v2, vcc, v2, s12
+    v_addc_co_u32         v3, vcc, v3, 0x0, vcc
+  end
+
+  // Repeat until branched out.
+  s_branch                L_FILL_PHASE_1_LOOP
+
+L_FILL_PHASE_1_DONE:
+
+  // ==============================
+  // Phase 2: Dword fill up to end.
+  // ==============================
+
+  // Compute dword stride across all threads.
+  s_lshl_b32              s12, s11, 0x2
+
+  // Compute phase destination address.
+  v_lshlrev_b32           v1, 0x2, v0
+  v_mov_b32               v3, s7
+  v_add_co_u32            v2, vcc, v1, s6
+  v_addc_co_u32           v3, vcc, v3, 0x0, vcc
+
+L_FILL_PHASE_2_LOOP:
+  // Mask off lanes (or branch out) after phase end.
+  v_cmp_lt_u64            vcc, v[2:3], s[8:9]
+  s_cbranch_vccz          L_FILL_PHASE_2_DONE
+  s_and_b64               exec, exec, vcc
+
+  // Write to/advance the destination address.
+  flat_store_dword        v[2:3], v4 sc0:1 sc1:1
+  v_add_co_u32            v2, vcc, v2, s12
+  v_addc_co_u32           v3, vcc, v3, 0x0, vcc
+
+  // Repeat until branched out.
+  s_branch                L_FILL_PHASE_2_LOOP
+
+L_FILL_PHASE_2_DONE:
+  s_endpgm
+end
 )");
 
 // Search kernel source for variable definition and return value.
@@ -522,6 +904,9 @@ BlitKernel::BlitKernel(core::Queue* queue)
       kernarg_async_(NULL),
       kernarg_async_mask_(0),
       kernarg_async_counter_(0),
+      bytes_queued_(0),
+      last_queued_(0),
+      pending_search_index_(0),
       num_cus_(0) {
   completion_signal_.handle = 0;
 }
@@ -531,6 +916,9 @@ BlitKernel::~BlitKernel() {}
 hsa_status_t BlitKernel::Initialize(const core::Agent& agent) {
   queue_bitmask_ = queue_->public_handle()->size - 1;
 
+  bytes_written_.resize(queue_->public_handle()->size);
+  memset(&bytes_written_[0], -1, bytes_written_.size() * sizeof(BytesWritten));
+
   hsa_status_t status = HSA::hsa_signal_create(1, 0, NULL, &completion_signal_);
   if (HSA_STATUS_SUCCESS != status) {
     return status;
@@ -594,9 +982,10 @@ hsa_status_t BlitKernel::SubmitLinearCopyCommand(void* dst, const void* src,
   HSA::hsa_signal_store_relaxed(completion_signal_, 1);
 
   std::vector<core::Signal*> dep_signals(0);
+  std::vector<core::Signal*> gang_signals(0);
 
   hsa_status_t stat = SubmitLinearCopyCommand(
-      dst, src, size, dep_signals, *core::Signal::Convert(completion_signal_));
+      dst, src, size, dep_signals, *core::Signal::Convert(completion_signal_), gang_signals);
 
   if (stat != HSA_STATUS_SUCCESS) {
     return stat;
@@ -614,12 +1003,19 @@ hsa_status_t BlitKernel::SubmitLinearCopyCommand(void* dst, const void* src,
 
 hsa_status_t BlitKernel::SubmitLinearCopyCommand(
     void* dst, const void* src, size_t size,
-    std::vector<core::Signal*>& dep_signals, core::Signal& out_signal) {
+    std::vector<core::Signal*>& dep_signals, core::Signal& out_signal,
+    std::vector<core::Signal*>& gang_signals) {
   // Reserve write index for barrier(s) + dispatch packet.
   const uint32_t num_barrier_packet = uint32_t((dep_signals.size() + 4) / 5);
   const uint32_t total_num_packet = num_barrier_packet + 1;
 
-  uint64_t write_index = AcquireWriteIndex(total_num_packet);
+  uint64_t write_index;
+  {
+    std::lock_guard<std::mutex> lock(reservation_lock_);
+    write_index = AcquireWriteIndex(total_num_packet);
+    RecordBlitHistory(size, write_index + total_num_packet - 1);
+  }
+
   uint64_t write_index_temp = write_index;
 
   // Insert barrier packets to handle dependent signals.
@@ -758,9 +1154,16 @@ hsa_status_t BlitKernel::SubmitLinearFillCommand(void* ptr, uint32_t value,
   // Submit dispatch packet.
   HSA::hsa_signal_store_relaxed(completion_signal_, 1);
 
-  uint64_t write_index = AcquireWriteIndex(1);
+  uint64_t write_index;
+  {
+    std::lock_guard<std::mutex> lock(reservation_lock_);
+    write_index = AcquireWriteIndex(1);
+    RecordBlitHistory(fill_size, write_index);
+  }
+
   PopulateQueue(write_index, uintptr_t(kernels_[KernelType::Fill].code_buf_),
                 args, num_workitems, completion_signal_);
+
   ReleaseWriteIndex(write_index, 1);
 
   // Wait for the packet to finish.
@@ -842,5 +1245,42 @@ BlitKernel::KernelArgs* BlitKernel::ObtainAsyncKernelCopyArg() {
   return arg;
 }
 
+void BlitKernel::RecordBlitHistory(uint64_t size, uint64_t index) {
+  uint64_t queued = bytes_queued_;
+  bytes_queued_ += size;
+  bytes_written_[index & queue_bitmask_].bytes = queued;
+  bytes_written_[index & queue_bitmask_].index = index;
+  last_queued_ = index;
+}
+
+uint64_t BlitKernel::PendingBytes() {
+  uint64_t read = queue_->LoadReadIndexRelaxed();
+  uint64_t index = pending_search_index_.load();
+  uint64_t last = last_queued_;
+  // If the last blit command has been run then the blit is empty.
+  if (read > last) return 0;
+
+  index = Max(index, read);
+  while (index <= last) {
+    // Ensure any record we use was not wrapped.
+    if (index == bytes_written_[index & queue_bitmask_].index) {
+      uint64_t ret = bytes_queued_ - bytes_written_[index & queue_bitmask_].bytes;
+
+      // Store max search index.
+      uint64_t old = pending_search_index_.load();
+      while (old < index) {
+        if (pending_search_index_.compare_exchange_strong(old, index)) break;
+      }
+
+      return ret;
+    }
+    index++;
+  }
+  debug_warning(false && "Race between PendingBytes and blit submission detected.");
+  // Zero is a valid return in this case since the command which was last when the search started is
+  // now complete.
+  return 0;
+}
+
 }  // namespace amd
 }  // namespace rocr
