@@ -57,8 +57,7 @@ namespace AMD {
 // Tracks aggregate size of system memory available on platform
 size_t MemoryRegion::max_sysmem_alloc_size_ = 0;
 
-void* MemoryRegion::AllocateKfdMemory(const HsaMemFlags& flag,
-                                      HSAuint32 node_id, size_t size) {
+void* MemoryRegion::AllocateKfdMemory(const HsaMemFlags& flag, HSAuint32 node_id, size_t size) {
   void* ret = NULL;
   const HSAKMT_STATUS status = hsaKmtAllocMemory(node_id, size, flag, &ret);
   return (status == HSAKMT_STATUS_SUCCESS) ? ret : NULL;
@@ -103,9 +102,8 @@ void MemoryRegion::MakeKfdMemoryUnresident(const void* ptr) {
 MemoryRegion::MemoryRegion(bool fine_grain, bool kernarg, bool full_profile,
                            bool extended_scope_fine_grain, core::Agent* owner,
                            const HsaMemoryProperties& mem_props)
-    : core::MemoryRegion(fine_grain, kernarg, full_profile, owner),
+    : core::MemoryRegion(fine_grain, kernarg, full_profile, extended_scope_fine_grain, owner),
       mem_props_(mem_props),
-      extended_scope_fine_grain_(extended_scope_fine_grain),
       max_single_alloc_size_(0),
       virtual_size_(0),
       fragment_allocator_(BlockAllocator(*this)) {
@@ -122,6 +120,12 @@ MemoryRegion::MemoryRegion(bool fine_grain, bool kernarg, bool full_profile,
   // coarse or fine grain or extended scope fine grain.
   mem_flag_.ui32.CoarseGrain = (fine_grain || extended_scope_fine_grain) ? 0 : 1;
 
+  // Extended scope fine-grained memory: Device scope atomics are promoted
+  // to system scope atomics. Non-compliant systems may require the
+  // application to perform device-specific actions, like HDP flushes,
+  // to achieve system-scope coherence
+  mem_flag_.ui32.ExtendedCoherent = (extended_scope_fine_grain) ? 1 : 0;
+
   if (IsLocalMemory()) {
     mem_flag_.ui32.PageSize = HSA_PAGE_SIZE_4KB;
     mem_flag_.ui32.NoSubstitute = 1;
@@ -131,19 +135,6 @@ MemoryRegion::MemoryRegion(bool fine_grain, bool kernarg, bool full_profile,
 
     virtual_size_ = kGpuVmSize;
 
-    // If memory region is extended scope fine grained
-    // mark the page table entries for this memory region
-    // as MTYPE_UC. Full read and write ordering are guaranteed
-    // to this address.
-    if (extended_scope_fine_grain) {
-      AMD::GpuAgent* agent_ =
-          const_cast<AMD::GpuAgent*>(reinterpret_cast<const AMD::GpuAgent*>(owner));
-      if (agent_->isa()->GetVersion() == core::Isa::Version(9, 4, 0) ||
-          agent_->isa()->GetVersion() == core::Isa::Version(9, 4, 1) ||
-          agent_->isa()->GetVersion() == core::Isa::Version(9, 4, 2))
-        mem_flag_.ui32.Uncached = 1;
-    }
-
   } else if (IsSystem()) {
     mem_flag_.ui32.PageSize = HSA_PAGE_SIZE_4KB;
     mem_flag_.ui32.NoSubstitute = 0;
@@ -176,13 +167,13 @@ MemoryRegion::MemoryRegion(bool fine_grain, bool kernarg, bool full_profile,
 
 MemoryRegion::~MemoryRegion() {}
 
-hsa_status_t MemoryRegion::Allocate(size_t& size, AllocateFlags alloc_flags, void** address) const {
+hsa_status_t MemoryRegion::Allocate(size_t& size, AllocateFlags alloc_flags, void** address, int agent_node_id) const {
   ScopedAcquire<KernelMutex> lock(&owner()->agent_memory_lock_);
-  return AllocateImpl(size, alloc_flags, address);
+  return AllocateImpl(size, alloc_flags, address, agent_node_id);
 }
 
 hsa_status_t MemoryRegion::AllocateImpl(size_t& size, AllocateFlags alloc_flags,
-                                        void** address) const {
+                                        void** address, int agent_node_id) const {
   if (address == NULL) {
     return HSA_STATUS_ERROR_INVALID_ARGUMENT;
   }
@@ -208,11 +199,17 @@ hsa_status_t MemoryRegion::AllocateImpl(size_t& size, AllocateFlags alloc_flags,
   if (IsSystem() && (alloc_flags & AllocateNonPaged))
       kmt_alloc_flags.ui32.NonPaged = 1;
 
+  // Allocating a memory handle for virtual memory
+  kmt_alloc_flags.ui32.NoAddress = !!(alloc_flags & AllocateMemoryOnly);
+
   // Allocate pseudo fine grain memory
   kmt_alloc_flags.ui32.CoarseGrain = (alloc_flags & AllocatePCIeRW ? 0 : kmt_alloc_flags.ui32.CoarseGrain);
+  kmt_alloc_flags.ui32.NoSubstitute = (alloc_flags & AllocatePinned ? 1 : kmt_alloc_flags.ui32.NoSubstitute);
+
+  kmt_alloc_flags.ui32.GTTAccess = (alloc_flags & AllocateGTTAccess ? 1 : kmt_alloc_flags.ui32.GTTAccess);
 
   // Only allow using the suballocator for ordinary VRAM.
-  if (IsLocalMemory()) {
+  if (IsLocalMemory() && !kmt_alloc_flags.ui32.NoAddress) {
     bool subAllocEnabled = !core::Runtime::runtime_singleton_->flag().disable_fragment_alloc();
     // Avoid modifying executable or queue allocations.
     bool useSubAlloc = subAllocEnabled;
@@ -230,14 +227,18 @@ hsa_status_t MemoryRegion::AllocateImpl(size_t& size, AllocateFlags alloc_flags,
     }
   }
 
+  const HSAuint32 node_id = (alloc_flags & AllocateGTTAccess) ? agent_node_id : owner()->node_id();
+
   // Allocate memory.
   // If it fails attempt to release memory from the block allocator and retry.
-  *address = AllocateKfdMemory(kmt_alloc_flags, owner()->node_id(), size);
+  *address = AllocateKfdMemory(kmt_alloc_flags, node_id, size);
   if (*address == nullptr) {
     owner()->Trim();
-    *address = AllocateKfdMemory(kmt_alloc_flags, owner()->node_id(), size);
+    *address = AllocateKfdMemory(kmt_alloc_flags, node_id, size);
   }
 
+  if (kmt_alloc_flags.ui32.NoAddress) return HSA_STATUS_SUCCESS;
+
   if (*address != nullptr) {
     // Commit the memory.
     // For system memory, on non-restricted allocation, map it to all GPUs. On
@@ -464,6 +465,22 @@ hsa_status_t MemoryRegion::GetPoolInfo(hsa_amd_memory_pool_info_t attribute,
       else
         return HSA_STATUS_ERROR_INVALID_ARGUMENT;
       break;
+    case HSA_AMD_MEMORY_POOL_INFO_RUNTIME_ALLOC_REC_GRANULE:
+      switch (mem_props_.HeapType) {
+        case HSA_HEAPTYPE_SYSTEM:
+          *((size_t*)value) = kPageSize_;
+          break;
+        case HSA_HEAPTYPE_FRAME_BUFFER_PRIVATE:
+        case HSA_HEAPTYPE_FRAME_BUFFER_PUBLIC:
+          *((size_t*)value) = core::Runtime::runtime_singleton_->flag().disable_fragment_alloc()
+              ? kPageSize_
+              : fragment_allocator_.default_block_size();
+          break;
+        default:
+          *((size_t*)value) = 0;
+          break;
+      }
+      break;
     default:
       return HSA_STATUS_ERROR_INVALID_ARGUMENT;
   }
@@ -752,7 +769,7 @@ void* MemoryRegion::BlockAllocator::alloc(size_t request_size, size_t& allocated
   size_t bsize = AlignUp(request_size, block_size());
 
   hsa_status_t err = region_.AllocateImpl(
-      bsize, core::MemoryRegion::AllocateRestrict | core::MemoryRegion::AllocateDirect, &ret);
+      bsize, core::MemoryRegion::AllocateRestrict | core::MemoryRegion::AllocateDirect, &ret, 0);
   if (err != HSA_STATUS_SUCCESS)
     throw AMD::hsa_exception(err, "MemoryRegion::BlockAllocator::alloc failed.");
   assert(ret != nullptr && "Region returned nullptr on success.");
