@@ -54,7 +54,7 @@ namespace rocr {
 namespace core {
 
 HsaEvent* InterruptSignal::EventPool::alloc() {
-  ScopedAcquire<KernelMutex> lock(&lock_);
+  ScopedAcquire<HybridMutex> lock(&lock_);
   if (events_.empty()) {
     if (!allEventsAllocated) {
       HsaEvent* evt = InterruptSignal::CreateEvent(HSA_EVENTTYPE_SIGNAL, false);
@@ -70,7 +70,7 @@ HsaEvent* InterruptSignal::EventPool::alloc() {
 
 void InterruptSignal::EventPool::free(HsaEvent* evt) {
   if (evt == nullptr) return;
-  ScopedAcquire<KernelMutex> lock(&lock_);
+  ScopedAcquire<HybridMutex> lock(&lock_);
   events_.push_back(unique_event_ptr(evt));
 }
 
@@ -89,6 +89,8 @@ HsaEvent* InterruptSignal::CreateEvent(HSA_EVENTTYPE type, bool manual_reset) {
     if (type == HSA_EVENTTYPE_MEMORY) {
       memset(&ret->EventData.EventData.MemoryAccessFault.Failure, 0,
              sizeof(HsaAccessAttributeFailure));
+    } else if (type == HSA_EVENTTYPE_HW_EXCEPTION) {
+      memset(&ret->EventData.EventData.HwException, 0, sizeof(HsaHwException));
     }
   }
 
