@@ -52,7 +52,7 @@
 namespace rocr {
 namespace core {
 
-KernelMutex Signal::ipcLock_;
+HybridMutex Signal::ipcLock_;
 std::map<decltype(hsa_signal_t::handle), Signal*> Signal::ipcMap_;
 
 void SharedSignalPool_t::clear() {
@@ -70,14 +70,14 @@ void SharedSignalPool_t::clear() {
 }
 
 SharedSignal* SharedSignalPool_t::alloc() {
-  ScopedAcquire<KernelMutex> lock(&lock_);
+  ScopedAcquire<HybridMutex> lock(&lock_);
   if (free_list_.empty()) {
     SharedSignal* block = reinterpret_cast<SharedSignal*>(
-        allocate_(block_size_ * sizeof(SharedSignal), __alignof(SharedSignal), 0));
+        allocate_(block_size_ * sizeof(SharedSignal), __alignof(SharedSignal), 0, 0));
     if (block == nullptr) {
       block_size_ = minblock_;
       block = reinterpret_cast<SharedSignal*>(
-          allocate_(block_size_ * sizeof(SharedSignal), __alignof(SharedSignal), 0));
+          allocate_(block_size_ * sizeof(SharedSignal), __alignof(SharedSignal), 0, 0));
       if (block == nullptr) throw std::bad_alloc();
     }
 
@@ -103,7 +103,7 @@ void SharedSignalPool_t::free(SharedSignal* ptr) {
   if (ptr == nullptr) return;
 
   ptr->~SharedSignal();
-  ScopedAcquire<KernelMutex> lock(&lock_);
+  ScopedAcquire<HybridMutex> lock(&lock_);
 
   ifdebug {
     bool valid = false;
@@ -128,7 +128,7 @@ LocalSignal::LocalSignal(hsa_signal_value_t initial_value, bool exportable)
 }
 
 void Signal::registerIpc() {
-  ScopedAcquire<KernelMutex> lock(&ipcLock_);
+  ScopedAcquire<HybridMutex> lock(&ipcLock_);
   auto handle = Convert(this);
   assert(ipcMap_.find(handle.handle) == ipcMap_.end() &&
          "Can't register the same IPC signal twice.");
@@ -136,7 +136,7 @@ void Signal::registerIpc() {
 }
 
 bool Signal::deregisterIpc() {
-  ScopedAcquire<KernelMutex> lock(&ipcLock_);
+  ScopedAcquire<HybridMutex> lock(&ipcLock_);
   if (refcount_ != 0) return false;
   auto handle = Convert(this);
   const auto& it = ipcMap_.find(handle.handle);
@@ -146,14 +146,14 @@ bool Signal::deregisterIpc() {
 }
 
 Signal* Signal::lookupIpc(hsa_signal_t signal) {
-  ScopedAcquire<KernelMutex> lock(&ipcLock_);
+  ScopedAcquire<HybridMutex> lock(&ipcLock_);
   const auto& it = ipcMap_.find(signal.handle);
   if (it == ipcMap_.end()) return nullptr;
   return it->second;
 }
 
 Signal* Signal::duplicateIpc(hsa_signal_t signal) {
-  ScopedAcquire<KernelMutex> lock(&ipcLock_);
+  ScopedAcquire<HybridMutex> lock(&ipcLock_);
   const auto& it = ipcMap_.find(signal.handle);
   if (it == ipcMap_.end()) return nullptr;
   it->second->refcount_++;
@@ -267,6 +267,9 @@ uint32_t Signal::WaitAny(uint32_t signal_count, const hsa_signal_t* hsa_signals,
           if (fault.Flags == HSA_EVENTID_MEMORY_FATAL_PROCESS) {
             return i;
           }
+        } else if (event_type == HSA_EVENTTYPE_HW_EXCEPTION) {
+          const HsaHwException& exception = signals[i]->EopEvent()->EventData.EventData.HwException;
+          if (exception.MemoryLost) return i;
         }
       }
 
