@@ -47,12 +47,14 @@
 #include <climits>
 #include <mutex>
 
+#include "core/inc/runtime.h"
 #include "core/inc/hsa_internal.h"
 #include "core/inc/hsa_ext_amd_impl.h"
 #include "resource.h"
 #include "image_manager_kv.h"
 #include "image_manager_ai.h"
 #include "image_manager_nv.h"
+#include "image_manager_gfx11.h"
 #include "device_info.h"
 
 namespace rocr {
@@ -108,7 +110,9 @@ hsa_status_t ImageRuntime::CreateImageManager(hsa_agent_t agent, void* data) {
 
     ImageManager* image_manager;
 
-    if (major_ver >= 10) {
+    if (major_ver >= 11) {
+      image_manager = new ImageManagerGfx11();
+    } else if (major_ver >= 10) {
       image_manager = new ImageManagerNv();
     } else if (major_ver >= 9) {
       image_manager = new ImageManagerAi();
@@ -394,6 +398,8 @@ hsa_status_t ImageRuntime::CreateImageHandle(
 
   image_manager(component)->PopulateImageSrd(*image);
 
+  if (core::Runtime::runtime_singleton_->flag().image_print_srd()) image->printSRD();
+
   image_handle.handle = image->Convert();
 
   return HSA_STATUS_SUCCESS;
@@ -431,6 +437,8 @@ hsa_status_t ImageRuntime::CreateImageHandleWithLayout(
     return err;
   }
 
+  if (core::Runtime::runtime_singleton_->flag().image_print_srd()) image->printSRD();
+
   image_handle.handle = image->Convert();
   return HSA_STATUS_SUCCESS;
 }
